#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-05-20

# based on https://github.com/allangood/check_mk/tree/master/plugins/squid by Allan GooD: allan.cassaro@gmail.com
#
# rewritten for cmk 2.x
#
# 2023-05-20: moved to ~/local/lib/check_mk/base/plugins/agent_based/utils
#             renamed from check_squid.include to squid.py
# 2023-12-02: fixed crash on empty section or missing value
#             added swap section
#             added file descriptors section

# ToDo: add support for multiple instances

from typing import Dict, Optional
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    check_levels,
    check_levels_predictive,
    Service,
    render
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
)


def parse_squid(string_table) -> Dict[str, float]:
    section = {}
    for line in string_table:
        entry, value = line[0].split('=')
        entry = entry.strip()
        try:
            value = float(value.split('/')[0].strip().split(' ')[0].split('%')[0])
        except ValueError:
            continue
        if 'kbytes' in entry:
            value = value * 1024 + 8  # change kbytes to bits
        section[entry] = value
    return section


register.agent_section(
    name='squid',
    parse_function=parse_squid,
)


def parse_squid_file_descriptors(string_table) -> Dict[str, float]:
    """
        Maximum number of file descriptors:   1024
        Available number of file descriptors: 1018
        Reserved number of file descriptors:   100

    Args:
        string_table:

    Returns:

    """
    section = {}
    for line in string_table:
        entry, value = line[0].split(':')
        entry = entry.lower().strip().replace(' ', '_')
        try:
            value = float(value.strip())
        except ValueError:
            continue
        section[entry] = value
    return section


register.agent_section(
    name='squid_file_descriptors',
    parse_function=parse_squid_file_descriptors,
)


def render_per_second(value: float) -> str:
    return f'{value:0.2f}/s'


def render_count(value: float) -> str:
    return f'{int(value)}'


squid_map = {
    'Client': {
        'client_http.requests': {
            'label': 'Requests',
            'levels_upper': 'client_requests',
            'metric_name': 'client_requests_per_second',
            'render_func': render_per_second,
        },
        'client_http.errors': {
            'label': 'Errors',
            'levels_upper': 'client_errors',
            'metric_name': 'client_errors_per_second',
            'render_func': render_per_second,
        },
        'client_http.hits': {
            'label': 'Cache Hits',
            'levels_upper': 'client_hits',
            'metric_name': 'client_hits_per_second',
            'render_func': render_per_second,
        },
        'client_http.kbytes_in': {
            'label': 'In',
            'levels_upper': 'client_bytes_in_out',
            'metric_name': 'if_in_bps',
            'render_func': render.networkbandwidth,
        },
        'client_http.kbytes_out': {
            'label': 'Out',
            'levels_upper': 'client_bytes_in_out',
            'metric_name': 'if_out_bps',
            'render_func': render.networkbandwidth,
        },
    },
    'Server': {
        'server.all.requests': {
            'label': 'Requests',
            'levels_upper': 'server_requests',
            'metric_name': 'server_all_requests',
            'render_func': render_per_second,
        },
        'server.all.errors': {
            'label': 'Errors',
            'levels_upper': 'server_errors',
            'metric_name': 'server_all_errors',
            'render_func': render_per_second,
        },
        'server.all.kbytes_in': {
            'label': 'In',
            'levels_upper': 'server_bytes_in_out',
            'metric_name': 'if_in_bps',
            'render_func': render.networkbandwidth,
        },
        'server.all.kbytes_out': {
            'label': 'Out',
            'levels_upper': 'server_bytes_in_out',
            'metric_name': 'if_out_bps',
            'render_func': render.networkbandwidth,
        },
    },
    'CPU': {
        'cpu_usage': {
            'label': 'Usage',
            'levels_upper': 'cpu_usage',
            'metric_name': 'util',
            'render_func': render.percent,
        },
        'cpu_time': {
            'label': 'Time',
            'levels_upper': 'cpu_time',
            'metric_name': 'cpu_time',
            'render_func': render.timespan,
        },
    },
    'DNS': {
        'dns.median_svc_time': {
            'label': 'Median response time',
            'levels_upper': 'dns_time',
            'metric_name': 'dns_median_svc_time',
            'render_func': render.timespan,
        }},
    'Swap': {
        'swap.outs': {
            'label': 'Out',
            'levels_upper': 'swap_out',
            'metric_name': 'swap_out',
            'render_func': render_per_second,
        },
        'swap.ins': {
            'label': 'In',
            'levels_upper': 'swap_in',
            'metric_name': 'swap_in',
            'render_func': render_per_second,
        },
        'swap.files_cleaned': {
            'label': 'Files cleaned',
            'levels_upper': 'swap_files_cleaned',
            'metric_name': 'swap_files_cleaned',
            'render_func': render_per_second,
        },
    },
    'File descriptors': {
        'maximum_number_of_file_descriptors': {
            'label': 'Maximum',
            'levels_upper': 'maximum_number_of_file_descriptors',
            'metric_name': 'maximum_number_of_file_descriptors',
            'render_func': render_count,
        },
        'available_number_of_file_descriptors': {
            'label': 'Available',
            'levels_upper': 'available_number_of_file_descriptors',
            'metric_name': 'available_number_of_file_descriptors',
            'render_func': render_count,
        },
        'reserved_number_of_file_descriptors': {
            'label': 'Reserved',
            'levels_upper': 'reserved_number_of_file_descriptors',
            'metric_name': 'reserved_number_of_file_descriptors',
            'render_func': render_count,
        },
    }
}


def discovery_squid(
        section_squid: Optional[Dict[str, float]],
        section_squid_file_descriptors: Optional[Dict[str, float]]
) -> DiscoveryResult:
    for item in squid_map.keys():
        yield Service(item=item)


def check_squid(
        item,
        params,
        section_squid: Optional[Dict[str, float]],
        section_squid_file_descriptors: Optional[Dict[str, float]],
) -> CheckResult:
    section_squid.update(section_squid_file_descriptors)
    for value in squid_map[item].keys():
        entry = squid_map[item][value]
        levels_upper = params.get(entry['levels_upper'], None)
        if section_squid.get(value) is not None:
            yield from check_levels_predictive(
                label=entry['label'],
                value=section_squid[value],
                levels=levels_upper,
                render_func=entry['render_func'],
                metric_name=entry['metric_name'],
            ) if isinstance(levels_upper, dict) else check_levels(
                label=entry['label'],
                value=section_squid[value],
                levels_upper=levels_upper,
                render_func=entry['render_func'],
                metric_name=entry['metric_name'],
            )


register.check_plugin(
    name='squid',
    service_name='Squid %s',
    sections=['squid', 'squid_file_descriptors'],
    discovery_function=discovery_squid,
    check_function=check_squid,
    check_default_parameters={
        # 'client_requests': (600, 800),
        # 'client_hits': (600, 800),
        # 'server_requests': (600, 800),
        'dns_time': (2.0, 4.0),
        'cpu_usage': (60.0, 80.0)
    },
    check_ruleset_name='squid',
)
